@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -593,8 +594,10 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
     } else
       in_cache = 1;
 
-    if(cached_dir == NULL)
+    if(cached_dir == NULL) {
+      free(cached_dir_base);
       return 0;
+    }
 
     p = 0;
 
@@ -611,9 +614,7 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
             memcpy(FileICB, &tmpICB, sizeof(tmpICB));
             found = 1;
           }
-          if(!UDFMapICB(device, tmpICB, &tmpFiletype, partition, &tmpFile))
-            return 0;
-
+          UDFMapICB(device, tmpICB, &tmpFiletype, partition, &tmpFile);
         } else {
           if( !strcasecmp( FileName, filename ) ) {
             memcpy(FileICB, &tmpICB, sizeof(tmpICB));
